﻿-- ********************************************************************************
-- Halo Minimalistic
-- A minimalistic halo range display.
-- By: Shenton
--
-- HaloMini.lua
-- ********************************************************************************

--[[-------------------------------------------------------------------------------
    Globals to locals
--]]-------------------------------------------------------------------------------

local _G = _G;
local print = print;
local pairs = pairs;
local type = type;
local select = select;
local IsItemInRange = IsItemInRange;
local CheckInteractDistance = CheckInteractDistance;
local UnitClass = UnitClass;
local UnitLevel = UnitLevel;
local UnitExists = UnitExists;
local UnitIsFriend = UnitIsFriend;
local SlashCmdList = SlashCmdList;
local GetScreenWidth = GetScreenWidth;
local GameTooltip = GameTooltip;
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory;
local GetActiveSpecGroup = GetActiveSpecGroup;
local GetSpecialization = GetSpecialization;
local GetSpecializationInfo = GetSpecializationInfo;
local GetTalentInfo = GetTalentInfo;

--[[-------------------------------------------------------------------------------
    Main object
--]]-------------------------------------------------------------------------------

local A = CreateFrame("Frame");
_G["HaloMinimalisticGlobal"] = A;

--[[-------------------------------------------------------------------------------
    Variables
--]]-------------------------------------------------------------------------------

-- Addon version
A.version = GetAddOnMetadata("HaloMini", "Version");

-- Default saved variables database
A.defaultDB =
{
    point = "CENTER",
    relativeTo = "UIParent",
    relativePoint = "CENTER",
    offX = 0,
    offY = 0,
    locked = 1,
    scale = 1,
    refreshRate = 0.5,
    specOne = 1,
    specTwo = 1,
    specOneMode = "TARGET",
    specTwoMode = "TARGET",
};

-- Localization var
local L;

--[[-------------------------------------------------------------------------------
    Methods
--]]-------------------------------------------------------------------------------

-- Print
function A:Print(text)
    print("|cff33ff99HaloMini:|r "..text);
end

-- Check if the player want Halo Minimalistic with the current spec
function A:HaloMiniWanted()
    if ( GetActiveSpecGroup() == 1 and A.db.specOne ) then
        return 1;
    end

    if ( GetActiveSpecGroup() == 2 and A.db.specTwo ) then
        return 1;
    end

    return nil;
end

-- Range check
-- 30651 Dertrok's First Wand -- 15 yards -- Harm
-- 10645 gnomish death ray -- 20 yards -- Harm
-- 24268 tailoring net (netherweave) -- 25 yards -- Harm
-- 10716 Gnomish Shrink Ray -- 30 yards -- Harm
-- 18904 Zorbin's Ultra-Shrinker -- 35 yards -- Harm
-- 4945 Faintly Glowing Skull -- 40 yards -- Harm -- No longer available in game (old quest reward, pre cata)
-- 33581 Vrykul Insult -- 40 yards -- Harm

-- 1251 Linen Bandage -- 15 yards -- Help
-- 21519 mistletoe -- 20 yards -- Help
-- 56247 Box of Crossbow Bolts -- 25 yards -- Help
-- 1180 Scroll of Stamina -- 30 yards -- Help
-- 18904 Zorbin's Ultra-Shrinker -- 35 yards -- Help
-- 38266 Rotund Relic -- 40 yards -- Help

-- CheckInteractDistance("unit", 1) -- 1 = inspect, 4 = follow -- both = 28 yards -- help & harm
function A:CheckRangeCascade()
    local r15, r20, r25, r30, r35, r40;

    if ( UnitIsFriend("player", A.modeUnit) ) then -- Friendly target
        r15 = IsItemInRange(1251, A.modeUnit);
        r20 = IsItemInRange(21519, A.modeUnit);
        r25 = IsItemInRange(56247, A.modeUnit);
        r30 = IsItemInRange(1180, A.modeUnit);
        r35 = IsItemInRange(18904, A.modeUnit);
        r40 = IsItemInRange(38266, A.modeUnit);
    else
        r15 = IsItemInRange(30651, A.modeUnit);
        r20 = IsItemInRange(10645, A.modeUnit);
        r25 = IsItemInRange(24268, A.modeUnit);
        r30 = IsItemInRange(10716, A.modeUnit);
        r35 = IsItemInRange(18904, A.modeUnit);
        r40 = IsItemInRange(33581, A.modeUnit);
    end

    -- Setting values to nil even if not in range and capable of using this item spell
    if ( r15 and r15 == 0 ) then r15 = nil; end
    if ( r20 and r20 == 0 ) then r20 = nil; end
    if ( r25 and r25 == 0 ) then r25 = nil; end
    if ( r30 and r30 == 0 ) then r30 = nil; end
    if ( r35 and r35 == 0 ) then r35 = nil; end
    if ( r40 and r40 == 0 ) then r40 = nil; end

    return r15, r20, r25, r30, r35, r40;
end

function A:CheckRangeDivineStar()
    local r15, r20, r25;

    if ( UnitIsFriend("player", A.modeUnit) ) then -- Friendly target
        r15 = IsItemInRange(1251, A.modeUnit);
        r20 = IsItemInRange(21519, A.modeUnit);
        r25 = IsItemInRange(56247, A.modeUnit);
    else
        r15 = IsItemInRange(30651, A.modeUnit);
        r20 = IsItemInRange(10645, A.modeUnit);
        r25 = IsItemInRange(24268, A.modeUnit);
    end

    -- Setting values to nil even if not in range and capable of using this item spell
    if ( r15 and r15 == 0 ) then r15 = nil; end
    if ( r20 and r20 == 0 ) then r20 = nil; end
    if ( r25 and r25 == 0 ) then r25 = nil; end

    return r15, r20, r25;
end

function A:CheckRangeHalo()
    local r15, r20, r25, r30;

    if ( UnitIsFriend("player", A.modeUnit) ) then -- Friendly target
        r15 = IsItemInRange(1251, A.modeUnit);
        r20 = IsItemInRange(21519, A.modeUnit);
        r25 = IsItemInRange(56247, A.modeUnit);
        r30 = IsItemInRange(1180, A.modeUnit);
    else
        r15 = IsItemInRange(30651, A.modeUnit);
        r20 = IsItemInRange(10645, A.modeUnit);
        r25 = IsItemInRange(24268, A.modeUnit);
        r30 = IsItemInRange(10716, A.modeUnit);
    end

    -- Setting values to nil even if not in range and capable of using this item spell
    if ( r15 and r15 == 0 ) then r15 = nil; end
    if ( r20 and r20 == 0 ) then r20 = nil; end
    if ( r25 and r25 == 0 ) then r25 = nil; end
    if ( r30 and r30 == 0 ) then r30 = nil; end

    return r15, r20, r25, r30;
end

-- Set the check range mode
function A:SetMode()
    if ( GetActiveSpecGroup() == 1 ) then
        if ( A.db.specOneMode == "TARGET" ) then
            A.modeUnit = "target";
            A:EnableUpdate();
        elseif ( A.db.specOneMode == "MOUSEOVER" ) then
            A.modeUnit = "mouseover";
            A:EnableUpdate();
        elseif ( A.db.specOneMode == "BOTH" ) then
            A:EnableUpdateBoth();
        end
    elseif ( GetActiveSpecGroup() == 2 ) then
        if ( A.db.specTwoMode == "TARGET" ) then
            A.modeUnit = "target";
            A:EnableUpdate();
        elseif ( A.db.specTwoMode == "MOUSEOVER" ) then
            A.modeUnit = "mouseover";
            A:EnableUpdate();
        elseif ( A.db.specTwoMode == "BOTH" ) then
            A:EnableUpdateBoth();
        end
    end
end

-- Set the main frame scale
function A:SetScale()
    A.frame:SetScale(A.db.scale);
end

-- Set the main frame position
function A:SetPos()
    A.frame:ClearAllPoints();
    A.frame:SetPoint(A.db.point, A.db.relativeTo, A.db.relativePoint, A.db.offX, A.db.offY);
end

-- Reset frame position
function A:ResetPos()
    A.db.point = "CENTER";
    A.db.relativeTo = "UIParent";
    A.db.relativePoint = "CENTER";
    A.db.offX = 0;
    A.db.offY = 0;

    A:SetPos();
end

-- Lock the frame
function A:Lock()
    -- Lock it
    A.frame:SetMovable(nil);
    A.frame:EnableMouse(nil);
    A.frame:SetScript("OnDragStart", nil);
    A.frame:SetScript("OnDragStop", nil);
    A.frame:SetScript("OnEnter", nil);
    A.frame:SetScript("OnLeave", nil);

    -- Enable OnUpdate script
    A:SetMode();

    -- Tell the player the frame is lock
    A:Print(L["LOCKED"]);
end

-- Unlock the frame
function A:Unlock()
    -- Disable OnUpdate script
    A.frame:SetScript("OnUpdate", nil);

    -- Set movable
    A.frame:SetMovable(1);
    A.frame:EnableMouse(1);
    A.frame:RegisterForDrag("LeftButton");
    A.frame:SetScript("OnDragStart", A.frame.StartMoving);
    A.frame:SetScript("OnDragStop", function()
        A.frame:StopMovingOrSizing();

        local point, relativeTo, relativePoint, offX, offY = A.frame:GetPoint(1);

        A.db.point = point;
        A.db.relativeTo = relativeTo;
        A.db.relativePoint = relativePoint;
        A.db.offX = offX;
        A.db.offY = offY;
    end);
    A.frame:SetScript("OnEnter", function()
        A:DisplayUnlockedTooltip();
    end);
    A.frame:SetScript("OnLeave", function()
        GameTooltip:Hide();
    end);

    -- Display arrows and green dot
    A.frame.middleGray:Hide();
    A.frame.middleGreen:Hide();
    A.frame.middleYellow:Hide();
    A.frame.middleRed:Hide();
    A.frame.middleInfo:Show();
    A.frame.left:Show();
    A.frame.right:Show();

    -- Tell the player the frame is unlock
    A:Print(L["UNLOCKED"]);
end

-- Toggle lock/unlock
function A:ToggleLock()
    if ( A.frame:IsMovable() ) then
        A:Lock();
    else
        A:Unlock();
    end
end

-- Display a warning tooltip when hovering unlocked main frame
function A:DisplayUnlockedTooltip()
    if ( A.frame:GetRight() >= ( GetScreenWidth() / 2 ) ) then
        GameTooltip:SetOwner(A.frame, "ANCHOR_LEFT");
    else
        GameTooltip:SetOwner(A.frame, "ANCHOR_RIGHT");
    end

    GameTooltip:AddDoubleLine("|cffffffffHalo Minimalistic", "|cff33ff99v"..A.version);
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine(L["TOOLTIP_UNLOCKED_INFO"], nil, nil, nil, 1);

    GameTooltip:Show();
end

-- Get a specializaion name
function A:GetSpecializationName(specNum)
    if ( GetSpecialization(false, false, specNum) ) then
        return select(2, GetSpecializationInfo(GetSpecialization(false, false, specNum), false, false));
    end

    return L["NONE"];
end

-- Set the SetFrame() method
function A:SetSetFrameMethod()
    local specGroup = GetActiveSpecGroup();

    if ( select(4, GetTalentInfo(6, 1, specGroup)) ) then -- Cascade
        A.SetFrame = A.SetFrameForCascade;
        A:Print(L["SETTING_FRAME_FOR"]:format(L["CASCADE"]));
        return;
    end

    if ( select(4, GetTalentInfo(6, 2, specGroup)) ) then -- Divine Star
        A.SetFrame = A.SetFrameForDivineStar;
        A:Print(L["SETTING_FRAME_FOR"]:format(L["DIVINE_STAR"]));
        return;
    end

    if ( select(4, GetTalentInfo(6, 3, specGroup)) ) then -- Halo
        A.SetFrame = A.SetFrameForHalo;
        A:Print(L["SETTING_FRAME_FOR"]:format(L["HALO"]));
        return;
    end

    A.SetFrame = A.SetFrameForHalo; -- Default to Halo
    A:Print(L["SETTING_FRAME_FOR"]:format(L["HALO"]));
end

-- Set main frame for Cascade
function A:SetFrameForCascade()
    -- Getting range status
    local r15, r20, r25, r30, r35, r40 = A:CheckRangeCascade();

    -- Setting layers
    if ( r15 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r15 and r20 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r20 and r25 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r25 and r30 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r30 and r35 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r35 and r40 ) then
        A.frame.left:Hide();
        A.frame.right:Hide();
        A.frame.middleGreen:Show();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Hide();
    elseif ( not r40 ) then
        A.frame.left:Hide();
        A.frame.right:Show();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Show();
    end
end

-- Set main frame for Divine Star
function A:SetFrameForDivineStar()
    -- Getting range status
    local r15, r20, r25 = A:CheckRangeDivineStar();

    -- Setting layers
    if ( r15 ) then
        A.frame.left:Hide();
        A.frame.right:Hide();
        A.frame.middleGreen:Show();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Hide();
    elseif ( not r15 and r20 ) then
        A.frame.left:Hide();
        A.frame.right:Hide();
        A.frame.middleGreen:Show();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Hide();
    elseif ( not r20 and r25 ) then
        A.frame.left:Hide();
        A.frame.right:Show();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r25 ) then
        A.frame.left:Hide();
        A.frame.right:Show();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Show();
    end
end

-- Set main frame for Halo
function A:SetFrameForHalo()
    -- Getting range status
    local r15, r20, r25, r30 = A:CheckRangeHalo();

    -- Setting layers
    if ( r15 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Show();
    elseif ( not r15 and r20 ) then
        A.frame.left:Show();
        A.frame.right:Hide();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r20 and r25 ) then
        A.frame.left:Hide();
        A.frame.right:Hide();
        A.frame.middleGreen:Show();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Hide();
    elseif ( not r25 and r30 ) then
        A.frame.left:Hide();
        A.frame.right:Show();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Show();
        A.frame.middleRed:Hide();
    elseif ( not r30 ) then
        A.frame.left:Hide();
        A.frame.right:Show();
        A.frame.middleGreen:Hide();
        A.frame.middleYellow:Hide();
        A.frame.middleRed:Show();
    end
end

--[[-------------------------------------------------------------------------------
    Events
--]]-------------------------------------------------------------------------------

A:RegisterEvent("ADDON_LOADED");
A:RegisterEvent("PLAYER_LOGIN");

A:SetScript("OnEvent", function(self, event, ...)
    local arg1 = ...;

    if ( event == "ADDON_LOADED" and arg1 == "HaloMini" ) then
        -- Setting default savedvar if empty
        if ( not _G["haloMiniCharacterDB"] ) then _G["haloMiniCharacterDB"] = {}; end

        for k,v in pairs(A.defaultDB) do
            if ( type(_G["haloMiniCharacterDB"][k]) == "nil" ) then
                _G["haloMiniCharacterDB"][k] = v;
            end
        end

        -- Adding savedvar to local object
        A.db = _G["haloMiniCharacterDB"];

        -- Setting localization var
        L = A.L;

        -- Unregister event
        A:UnregisterEvent("ADDON_LOADED");
    elseif ( event == "PLAYER_LOGIN" ) then
        if ( select(3, UnitClass("player")) == 5 and UnitLevel("player") >= 90 ) then -- It's a level 90+ priest
            -- Setting frame and layers objects to local object
            A.frame = _G["HaloMiniFrame"];
            A.frame.middleGray = _G["HaloMiniFrameMiddleGray"];
            A.frame.middleGreen = _G["HaloMiniFrameMiddleGreen"];
            A.frame.middleYellow = _G["HaloMiniFrameMiddleYellow"];
            A.frame.middleRed = _G["HaloMiniFrameMiddleRed"];
            A.frame.middleInfo = _G["HaloMiniFrameMiddleInfo"];
            A.frame.left = _G["HaloMiniFrameLeft"];
            A.frame.right = _G["HaloMiniFrameRight"];

            -- Setting slash commands
            A:SetSlashCommands();

            -- Adding options panel to Blizzard interface options panel
            A:CreateOptionsPanel();

            -- On update value var
            A.frame.lastUpdate = 0;

            -- Main frame setter method
            A:SetSetFrameMethod();

            -- Starting main OnUpdate script (Frame is hidden atm), and setting mode
            A:SetMode();

            -- Registering events we only want to fire when everything is loaded
            A:RegisterEvent("PLAYER_ENTERING_WORLD");
            A:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
            A:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
            A:RegisterEvent("PET_BATTLE_OPENING_START");
            A:RegisterEvent("PET_BATTLE_CLOSE");
            A:RegisterEvent("CHARACTER_POINTS_CHANGED");

            -- Unregister event
            A:UnregisterEvent("PLAYER_LOGIN");
        else -- Not a priest
            A:Print(L["NOT_ON_PRIEST"]);
            A:UnregisterAllEvents();
            A = nil;
        end
    elseif ( event == "PLAYER_ENTERING_WORLD" ) then
        -- Setting options
        A:SetScale();
        A:SetPos();
        if ( not A.db.locked ) then A:Unlock(); end
        if ( A:HaloMiniWanted() ) then A.frame:Show(); end

        -- Unregistering event we only need it on login/reload
        A:UnregisterEvent("PLAYER_ENTERING_WORLD");
    elseif ( event == "ACTIVE_TALENT_GROUP_CHANGED" ) then
        -- Player switched specialisation, display Halo Minimalistic?
        if ( A:HaloMiniWanted() ) then
            A:SetSetFrameMethod();
            A:SetMode();
            A.frame:Show();
        else
            A.frame:Hide();
        end
    elseif ( event == "PLAYER_SPECIALIZATION_CHANGED" ) then
        -- Fire when player learn/unlearn a specialization
        A.optionsPanel.frames.specOneEnable.tooltipRequirement = L["ENABLE_WITH_SPEC_1_DESC"]:format(A:GetSpecializationName(1));
        A.optionsPanel.frames.specTwoEnable.tooltipRequirement = L["ENABLE_WITH_SPEC_2_DESC"]:format(A:GetSpecializationName(2));
    elseif ( event == "PET_BATTLE_OPENING_START" ) then
        A.frame:Hide();
    elseif ( event == "PET_BATTLE_CLOSE" ) then
        if ( A:HaloMiniWanted() ) then A.frame:Show(); end
    elseif ( event == "CHARACTER_POINTS_CHANGED" ) then
        A:SetSetFrameMethod();
    end
end);

--[[-------------------------------------------------------------------------------
    OnUpdate scripts
--]]-------------------------------------------------------------------------------

-- Main OnUpdate script - Modes: target or mouseover
function A:EnableUpdate()
    A.frame:SetScript("OnUpdate", function(self, elapsed)
        self.lastUpdate = self.lastUpdate + elapsed;

        if ( self.lastUpdate > A.db.refreshRate ) then
            if ( UnitExists(A.modeUnit) ) then -- Got a target or mouseover
                A:SetFrame();
                A.frame.middleGray:Hide();
            else -- No target or mouseover
                A.frame.middleGray:Show();
                A.frame.middleGreen:Hide();
                A.frame.middleYellow:Hide();
                A.frame.middleRed:Hide();
                A.frame.left:Hide();
                A.frame.right:Hide();
            end

            A.frame.middleInfo:Hide();
            self.lastUpdate = 0;
        end
    end);
end

-- Main OnUpdate script - Mode: Both
function A:EnableUpdateBoth()
    A.frame:SetScript("OnUpdate", function(self, elapsed)
        self.lastUpdate = self.lastUpdate + elapsed;

        if ( self.lastUpdate > A.db.refreshRate ) then
            if ( UnitExists("mouseover") ) then -- Got a mouseover
                A.modeUnit = "mouseover";
                A:SetFrame();
                A.frame.middleGray:Hide();
            elseif ( UnitExists("target") ) then -- Got a target
                A.modeUnit = "target";
                A:SetFrame();
                A.frame.middleGray:Hide();
            else -- No target or mouseover
                A.frame.middleGray:Show();
                A.frame.middleGreen:Hide();
                A.frame.middleYellow:Hide();
                A.frame.middleRed:Hide();
                A.frame.left:Hide();
                A.frame.right:Hide();
            end

            A.frame.middleInfo:Hide();
            self.lastUpdate = 0;
        end
    end);
end

--[[-------------------------------------------------------------------------------
    Slash command
--]]-------------------------------------------------------------------------------

function A:SetSlashCommands()
    _G["SLASH_HALOMINI1"], _G["SLASH_HALOMINI2"] = "/hm", "/halomini";
    function SlashCmdList.HALOMINI(args)
        if ( args == "" ) then
            A:ToggleLock();
        elseif ( args == "wipedb" ) then
            for k,v in pairs(A.defaultDB) do
                A.db[k] = v;
            end

            A:Print(L["DATABASE_WIPED"]);
        elseif ( args == "config" ) then
            InterfaceOptionsFrame_OpenToCategory(A.optionsPanel);
        elseif ( args == "reset" ) then
            A:ResetPos();
        else
            A:Print(L["COMMAND_1"]);
            A:Print(L["COMMAND_2"]);
            A:Print(L["COMMAND_3"]);
            --A:Print(L["COMMAND_4"]);
        end
    end
end
